/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: RecorderUIUtility.java,v 1.4 2005/02/16 22:20:18 qiyanli Exp $
 * 
 * Contributors: 
 * IBM Corporation - initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.internal.execution.recorder.ui;

import java.text.MessageFormat;

import org.eclipse.hyades.execution.recorder.RecorderPlugin;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * UI utility for laying out and creating controls
 * @author Ernest Jessee
 */
public class RecorderUIUtility
{

	
	/**
	 * exception thrown by the RecorderHarnessUIUtility class when problems occur while createing or laying out various controls.
	 */
	public static class RecorderHarnessUIException extends Exception
	{
	
		
		public RecorderHarnessUIException()
		{
			super();
		}

		/**
		 * @param message
		 */
		public RecorderHarnessUIException(String message)
		{
			super(message);
		}	
		

	}
	
	
	
	/**
	 * Utility method for laying out a control in a form layout by specifying percentage-of-parent-width and border regions
	 *  If the parent of the control is not a FormLayout, an exception will be thrown
	 * @param Control control - the control to be layed out
	 * @param intpixelsToTop - pixels from top of the parent control to the parent of the control of focus
	 * @param int verticalSpanPixels - the vertical span of the control being layed out
	 * @param int leftPercentWidth - percentage of total parent width from left for left of target
	 * @param int rightPercentWidth - percentage of total parent width from left for right of target 
	 * @param int leftBorder - pixels for left border region
	 * @param int bottomBorder - pixels for bottom border region
	 * @param int topBorder - pixels for top border region
	 * @param int rightBorder - pixels for bottom border region
	 * @throws RecorderHarnessUIException
	 */
	public static void layoutCompositelInFormLayout(Control composite,
								 int pixelsToTop,
								 int verticalSpanPixels,
								 int leftPercentWidth, 
								 int rightPercentWidth, 
								 int leftBorder, 
								 int bottomBorder,
								 int topBorder, 
								 int rightBorder) throws RecorderHarnessUIException
	{
		if(!FormLayout.class.isInstance(composite.getParent().getLayout()))
		{
			String[] exceptionName = {"org.eclipse.swt.layout.FormLayout"}; //$NON-NLS-1$
			String errorMessage = MessageFormat.format(RecorderPlugin.getResourceString("RecorderUIUtility.INCORRECT_LAYOUT_MESSAGE_FORMAT_STRING"),exceptionName); //$NON-NLS-1$
			throw new RecorderHarnessUIException(errorMessage);
		}
		
		FormData formData = new FormData();
		formData.left = new FormAttachment(leftPercentWidth,leftBorder);
		formData.right = new FormAttachment(rightPercentWidth,-rightBorder);
		formData.top = new FormAttachment(0,pixelsToTop+topBorder);
		formData.bottom = new FormAttachment(0,pixelsToTop+verticalSpanPixels-bottomBorder);
		composite.setLayoutData(formData);
	}
	
	
	/**
	 * positions a control in a formlayout using only border specifications
	 * @param Control control
	 * @param int topBorder
	 * @param int leftBorder
	 * @param int bottomBorder
	 * @param int rightBorder
	 * @throws RecorderHarnessUIException
	 */
	public static void layoutControlInFormLayout(Control control,
											 int topBorder, 
											 int leftBorder, 
											 int bottomBorder, 
											 int rightBorder) throws RecorderHarnessUIException
	{
		if(!FormLayout.class.isInstance(control.getParent().getLayout()))
		{
			String[] exceptionName = {"org.eclipse.swt.layout.FormLayout"}; //$NON-NLS-1$
			String errorMessage = MessageFormat.format(RecorderPlugin.getResourceString("RecorderUIUtility.INCORRECT_LAYOUT_MESSAGE_FORMAT_STRING"),exceptionName); //$NON-NLS-1$
			throw new RecorderHarnessUIException(errorMessage);
		}
	
		FormData formData = new FormData();
		formData.left = new FormAttachment(0,leftBorder);
		formData.right = new FormAttachment(100,-rightBorder);
		formData.top = new FormAttachment(0,topBorder);
		formData.bottom = new FormAttachment(100,-bottomBorder);
				
		control.setLayoutData(formData);
	}
	


	
	/**
	 * method for laying out a control in a grid layout by providing columnSpan and any "or'ed" combination of GridData options
	 * @param Control control
	 * @param int columnSpan
	 * @param int gridDataOptions
	 * @throws RecorderHarnessUIException
	 */
	public static void layoutControlInGrid(Control control, int columnSpan, int gridDataOptions) throws RecorderHarnessUIException
	{
		if(!GridLayout.class.isInstance(control.getParent().getLayout()))
		{
			String[] exceptionName = {"org.eclipse.swt.layout.GridLayout"}; //$NON-NLS-1$
			String errorMessage = MessageFormat.format(RecorderPlugin.getResourceString("RecorderUIUtility.INCORRECT_LAYOUT_MESSAGE_FORMAT_STRING"),exceptionName); //$NON-NLS-1$
			throw new RecorderHarnessUIException(errorMessage);
		}
		GridData gridData = new GridData(gridDataOptions);
		gridData.horizontalSpan=columnSpan;
		control.setLayoutData(gridData);	
	}



	/**
	 * method for assigning a FormLayoout to a composite
	 * @param Composite parent
	 */	
	public static void assignFormLayout(Composite parent)
	{
		FormLayout layout = new FormLayout();
		layout.marginHeight=0;
		layout.marginWidth=0;
		parent.setLayout(layout);
	}




	/**
	 * method for assigning a GridLayout to a Composite
	 * @param Composite compsite
	 * @param int numberColumns
	 * @param int columnsEqual	 
	 */
	public static void assignGridLayout(Composite compsite, int numberColumns, boolean columnsEqual)
	{
		GridLayout gridLayout = new GridLayout(numberColumns,columnsEqual);
		compsite.setLayout(gridLayout);
	}




	/**
	 * method for creating a label
	 * @param Composite parent
	 * @param String labelText
	 * @param Font font
	 * @return Label
	 */
	public static Label createLabel(Composite parent, String labelText, Font font)
	{
		Label label = new Label(parent,SWT.WRAP);
		label.setText(labelText);
		
		if(font!=null)
			label.setFont(font);
		
		return label;
	}




	/**
	 * method for creating an Edit Control
	 * @param Composite parent
	 * @param String defaultText
	 * @param Font font
	 * @return Text
	 */
	public static Text createEditControl(Composite parent, String defaultText, Font font)
	{
		Text text = new Text(parent,SWT.BORDER);
		text.setText(defaultText);
		
		if(font!=null)
			text.setFont(font);
			
		return text;
	}
	
	
	
	/**
	 * method for creating a group
	 * @param Composite controlArea
	 * @param String groupDisplayName
	 * @return Group
	 */	
	public static Group createGroup(Composite controlArea, String groupDisplayName)
	{
		Group recorderHostGroup = new Group(controlArea,SWT.SHADOW_ETCHED_IN);
	
		RecorderUIUtility.assignFormLayout(controlArea);
			
		recorderHostGroup.setText(groupDisplayName);
	
		return recorderHostGroup;
	}
	
	
	/**
	 * method for creating a Button by passing a parent composite, button text, and any "or-ed" SWT options.
	 * @param Composite parent
	 * @param String text
	 * @param int swtOptions
	 * @return Button
	 */
	public static Button createButton(Composite parent, String text, int swtOptions)
	{
		Button button = new Button(parent,swtOptions);
		button.setText(text);
		return button;
	
	}
}
