/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: IComponent.java,v 1.8 2005/02/16 22:20:29 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.logging.adapter;
import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.w3c.dom.Element;
/**
 * <b>Describes component behavior</b>
 * Main role of the component is to provide dynamic managability.
 * 
 * @version 0.1
 * 
 */
public interface IComponent
{
	/**
	 * Method init.
	 * Sets all the context configuration properties based on the context config.
	 * @return boolean if init success
	 */
	boolean init();
	/**
	 * Method start. Starts component activity
	 */
	void start();
	/**
	 * Method stop. Stops the component activity
	 */
	void stop();
	/**
	 * Method update. Sets the component config with a root Element
	 * and then request the component to update based on that Element
	 * @param config string of config
	 * @throws AdapterInvalidConfig
	 */
	void update(Element config) throws AdapterInvalidConfig;
	/**
	 * Method update. By reading the current component config, sets the properties of 
	 * the component. Subclasses should invoke parent update() to ensure proper behaviour.
	 * @return
	 */
	void update() throws AdapterInvalidConfig;
	//=====basic getters and setters ==========================================
	/**
	 * Logging method.  
	 * <p>
	 * The Common Base Event is logged based on its severity.
	 * 
	 * @param event The Common Base Event to be logged.
	 */
	public void log(CommonBaseEvent event);
	/**
	 * Logging method.
	 * 
	 * @param event The Common Base Event to be logged.
	 * @param level The severity of the  Common Base Event to be logged.
	 * @deprecated Set the severity on the Common Base Event and use the <code>log(CommonBaseEvent event)</code> API.
	 */
	public void log(CommonBaseEvent event, short level);
	/**
	 * Method getContextConfiguration.
	 * @return Element - current config of the component
	 */
	Element getContextConfiguration();
	/**
	 * Sets the context config.
	 * @param config The configuration to set
	 */
	public void setContextConfiguration(Element newConfiguration);
	/**
	 * Method getConfiguration.
	 * @return Element - current config of the component
	 */
	Element getConfiguration();
	/**
	 * Sets the config.
	 * @param config The configuration to set
	 */
	public void setConfiguration(Element newConfiguration);
	/**
	 * Method returns the components this component manages
	 * @return org.eclipse.hyades.logging.adapter.IComponent[]
	 */
	IComponent[] getComponents();
	
	/**
	 * Method sets the list of components this component manages.  This component
	 * will now be considered the parent of each element it manages.
	 */
	void setComponents(IComponent[] newComponents);
	/**
	 * Method returns loggin level of the component
	 * @return short
	 */
	short getLoggingLevel();
	/**
	 * Method sets logging level of the component
	 * @return void
	 */
	void setLoggingLevel(short newLevel);
	
	/**
	 * Method returns name of the component
	 * @return String
	 */
	String getName();
	/**
	 * Method sets name of the component
	 * @return String
	 */
	void setName(String newName);
	/**
	 * Method returns the role of the component
	 * @return String
	 */
	String getRole();
	/**
	 * Method sets the role of the component
	 * @return String
	 */
	void setRole(String newRole);
	/**
	 * Method returns the component description
	 * @return String
	 */
	String getDescription();
	/**
	 * Method sets the description of this component
	 * @return String
	 */
	void setDescription(String newDescription);
	/**
	 * Method returns the uniqueID of this component instance
	 * @return String
	 */
	String getUniqueID();
	/**
	 * Method sets the uniqueID of this component instance
	 * @return String
	 */
	void setUniqueID(String newUniqueID);
	/**
	 * Method returns the executable class name
	 * @return String
	 */
	String getExecutableClassName();
	/**
	 * Method sets executable class name
	 * @return String
	 */
	void setExecutableClassName(String newExecutableClassName);
	/**
	 * Method returns the role version
	 * @return String
	 */
	String getRoleVersion();
	/**
	 * Method sets role version
	 * @return String
	 */
	void setRoleVersion(String newRoleVersion);
	/**
	 * Method returns role creation date
	 * @return String
	 */
	String getRoleCreationDate();
	/**
	 * Method sets role creation time
	 * @return String
	 */
	void setRoleCreationDate(String newCreationDate);
	/**
	 * Method returns the version specific role description
	 * @return String
	 */
	String getRoleVersionDescription();
	/**
	 * Method sets the version specific role description
	 * @return String
	 */
	void setRoleVersionDescription(String newRoleVersionDescription);
	/**
	 * Method returns the version specific role description
	 * @return String
	 */
	String getImplementationVersion();
	/**
	 * Method sets the implementation version in the form of x.y.z where 
	 * x is a major release level, which may have api changes
	 * y is a minor release level with no api changes
	 * z is an optional fix level with no api changes
	 * @return String
	 */
	void setImplementationVersion(String newImplementationVersion);
	/**
	 * Method returns implementation creation date
	 * @return String
	 */
	String getImplementationCreationDate();
	/**
	 * Method sets implementation creation date
	 * @return String
	 */
	void setImplemenationCreationDate(String newImplementationCreationDate);
	/**
	 * Method returns implementation version description
	 * @return String
	 */
	String getImplementationVersionDescription();
	/**
	 * Method sets implementation version description
	 * @return String
	 */
	void setImplementationVersionDescription(String newImplementationVersionDescription);

	
	/**
	 * Retrieve the logger for this adapter.  This cannot be called until the Component has
	 * been initialized by the init() method.
	 */
	public IProcessUnit getLogger();
	
	/**
	 * The logger will be set by the context during initialization.
	 * @param logger the logger for this component to use
	 */
	public void setLogger(IProcessUnit logger);
	
	
	/**
	 * Components are arranged in a heirarchy.  Sets the parent component of
	 * this component.  
	 * @param parent the parent component in the hierarchy
	 */
	public void setParent(IComponent parent);
	
	/**
	 * Components are arranged in a heirarchy.  Gets the parent component of
	 * this component.  
	 */
	public IComponent getParent();
}
