package org.eclipse.hyades.logging.events;

import java.util.List;

import org.eclipse.hyades.logging.events.exceptions.UnsupportedVersionException;

/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: ICommonBaseEvent.java,v 1.18 2005/02/16 22:20:48 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

/**
 * This is the interface that embodies the Common Base Event as described in 
 * "Canonical Situation Data Format: The Common Base Event V1.0.1 
 * ACAB.BO0301.0.1".  
 * 
 * 
 * @author Richard K. Duggan
 * @author Paul E. Slauenwhite
 * @version 1.0.1
 * @since 1.0
 * @deprecated This class is deprecated as of 2004-05-17. Use the corresponding class available in org.eclipse.hyades.logging.events.cbe*
 */
public interface ICommonBaseEvent extends org.eclipse.hyades.logging.core.ICommonBaseEvent {

    /**
     * Public static string version representation of the v1.0 Common Base Event implementation.
     */
    public static String VERSION_1_0 = "1.0";

    /**
     * Public static string version representation of the v1.0.1 Common Base Event implementation.
     */
    public static String VERSION_1_0_1 = "1.0.1";

    /**
     * Indicative of an event that does not need to be processed immediately.
     * 
     * @see #getPriority()
     */
    public static short PRIORITY_LOW = 10;

    /**
     * Indicative of an event of average importance.
     * 
     * @see #getPriority()
     */
    public static short PRIORITY_MEDIUM = 50;

    /**
     * Indicative of an important event that requires attention.
     * 
     * @see #getPriority()
     */
    public static short PRIORITY_HIGH = 70;

    /**
     * Information SHOULD be used for cases when the event only contains 
     * general information and is not reporting an error.
     * 
     * @see #getSeverity()
     */
    public static short SEVERITY_UNKNOWN = 0;

    /**
     * Information SHOULD be used for cases when the event only contains 
     * general information and is not reporting an error.
     * 
     * @see #getSeverity()
     */
    public static short SEVERITY_INFORMATION = 10;

    /**
     * Harmless SHOULD be used for cases when the error event has no effect 
     * on the normal operation of the resource.
     * 
     * @see #getSeverity()
     */
    public static short SEVERITY_HARMLESS = 20;

    /**
     * Warning SHOULD be used when it is appropriate to let the user decide 
     * if an action is needed in response to the event.
     * 
     * @see #getSeverity()
     */
    public static short SEVERITY_WARNING = 30;

    /**
     * Minor SHOULD be used to indicate that action is needed, but the 
     * situation is not serious at this time.
     * 
     * @see #getSeverity()
     */
    public static short SEVERITY_MINOR = 40;

    /**
     * Critical SHOULD be used to indicate that an immediate action is needed 
     * and the scope is broad (perhaps an imminent outage to a critical resource 
     * will result).
     * 
     * @see #getSeverity()
     */
    public static short SEVERITY_CRITICAL = 50;

    /**
     * Fatal SHOULD be used to indicate that an error occurred, but it is too 
     * late to take remedial action.
     * 
     * @see #getSeverity()
     */
    public static short SEVERITY_FATAL = 60;

    /**
     * Add a context that this event is referencing. See the
     * ContextDataElement definition (described on page 28) for details.
     * This is an OPTIONAL property.
     * 
     */
    public void addContextDataElement(IContextDataElement element);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     * 
     */
    public void addExtendedDataElement(IExtendedDataElement element);

    /**
     * This property allows for the grouping of events. This property is a
     * complex type that is made up of globalInstanceIds identifying the associated
     * events along with a type field describing the type of association that is
     * represented by the name of the association. Reserved keyword values include:
     * 1. Contain 
     * 2. CausedBy
     * 3. Cleared
     * 4. MultiPart
     * 5. Correlated
     * This is an OPTIONAL property.
     */
    public void addAssociatedEvent(IAssociatedEvent event);

    /**
     * Returns the ContextDataElements
     * 
     * An array of contexts that this event is referencing. See the
     * ContextDataElement definition (described on page 28) for details.
     * This is an OPTIONAL property.
     * 
     * @return List of Context Data Elements, otherwise an empty list.
     */
    public List getContextDataElements();

    /**
     * Returns the ExtendedDataElements associated with this event.
     * 
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     * 
     * @return List of Extended Data Elements, otherwise an empty list.
     */
    public List getExtendedDataElements();

    /**
     * Returns the Associated events.
     * 
     * This property allows for the grouping of events. This property is a
     * complex type that is made up of globalInstanceIds identifying the associated
     * events along with a type field describing the type of association that is
     * represented by the name of the association. Reserved keyword values include:
     * 1. Contain 
     * 2. CausedBy
     * 3. Cleared
     * 4. MultiPart
     * 5. Correlated
     * This is an OPTIONAL property.
     * 
     * @return List of Associated Events, otherwise an empty list.
     */
    public List getAssociatedEvents();

    /**
     * Get a named a context that this event is referencing. See the
     * ContextDataElement definition (described on page 28) for details.
     * 
     * @param name the name of the element to be found
     * 
     * @return IContextDataElement if the named element exists otherwise null is returned
     */
    public IContextDataElement getContextDataElement(String name);

    /**
     * Get a named extended data element that this event is referencing
     * 
     * @param name the name of the element to be found
     * 
     * @return IExtendedDataElement if the named element exists otherwise null is returned
     */
    public IExtendedDataElement getExtendedDataElement(String name);

    /**
     * Removes a named a context that this event is referencing. See the
     * ContextDataElement definition (described on page 28) for details.
     * 
     * @param name the name of the element to be removed
     */
    public void removeContextDataElement(String name);

    /**
     * Remove a named extended data element that this event is referencing 
     * 
     * @param name the name of the element to be removed
     */
    public void removeExtendedDataElement(String name);

    /**
     * Removes all IAssociatedEvents that this event is referencing 
     */
    public void clearAssociatedEvents();

    /**
     * Removes all IContextDataElements that this event is referencing 
     */
    public void clearContextDataElements();

    /**
     * Removes all IExtenedDataElements that this event is referencing 
     */
    public void clearExtendedDataElements();

    /**
     * Add an any element that this event is referencing. 
     * This is an OPTIONAL property.
     * 
     * @since 1.0 
     */
    public void addAnyElement(ICommonBaseEventAnyElement element);

    /**
     * Get a named a any element that this event is referencing. 
     * 
     * @param name the name of the element to be found
     * @return ICommonBaseEventAnyElement if the named element exists otherwise null is returned
     * @since 1.0 
     */
    public ICommonBaseEventAnyElement getAnyElement(String name);

    /**
     * Returns the ICommonBaseEventAnyElements
     * 
     * An array of any elements that this event is referencing. 
     * This is an OPTIONAL property.
     * 
     * @return List of Any Elements, otherwise an empty list.
     * @since 1.0 
     */
    public List getAnyElements();

    /**
     * Removes a named a any element that this event is referencing. 
     * 
     * @param name the name of the element to be removed
     * @since 1.0 
     */
    public void removeAnyElement(String name);

    /**
     * Removes all ICommonBaseEventAnyElements that this event is referencing 
     * @since 1.0 
     */
    public void clearAnyElements();

    /**
     * Returns the creationTime.
     * 
     * The date-time when the event was issued. The value MUST be as defined by 
     * the XML Schema dateTime data type. The value of the creationTime MUST provide
     * granularity as precisely as the generating platform allows. This is a REQUIRED
     *  property.
     * 
     * @return String
     */
    public String getCreationTime();

    /**
     * Returns the creationTime as a long holding the milliseconds since Jan 1, 1970.
     * 
     * @return long
     */
    public long getCreationTimeAsLong();

    /**
     * Returns the elapsedTime.
     * This is the time interval or the elapsed time for the number of occurrences
     * of a given event type that is specified by the repeatCount property. This
     * value indicates the duration of the time within which the repeated events 
     * were observed. This is an OPTIONAL property.  However, 
     * if repeatCount is specified you MUST specify a value for elapsedTime.
     * The default value is 0, which is a valid value for this property.
     * For this reason the '{@link org.eclipse.hyades.logging.events.ICommonBaseEvent#isSetElapsedTime <em>isSetElapsedTime()</em>}' 
     * method should always be called before calling this method.  This allows the user to 
     * distinguish the default value from a value that has been set to 0.
     * 
     * @return long
     */
    public long getElapsedTime();

    /**
     * Returns the extensionName.
     * 
     * The name of an event class (or element in XML) that this event represents
     * (e.g., CommonBaseEventImpl). The name is indicative of any additional elements
     * expected to be present within the event. This is a OPTIONAL property.  If 
     * the value specified is null, then its value is assumed to be 'CommonBaseEventImpl'.
     * The maximum string length for extensionName MUST not exceed 64 bytes.
     * 
     * @return String
     */
    public String getExtensionName();

    /**
     * Returns the gloablInstanceId.
     * 
     * The primary identifier for the event. This property MUST be globally unique
     * and MAY be used as the primary key for the event. This is an OPTIONAL property.
     * However, once this value is set it MUST never be changed. The recommend value 
     * is either a 128 bit or 256 bit Globally Unique Id (represented as hex string).
     * 
     * @return String
     */
    public String getGlobalInstanceId();

    /**
     * Returns the localInstanceId.
     * 
     * A source supplied event identifier. There is no guarantee that this value 
     * is globally unique. This is an OPTIONAL property. The maximum string length
     * for localInstanceId MUST not exceed 64 bytes.
     * 
     * @return String
     */
    public String getLocalInstanceId();

    /**
     * Returns the msg.
     * 
     * he text accompanying the event. This is typically the resolved message 
     * string in human readable format rendered for a specific locale. This is 
     * an OPTIONAL property.  The maximum string length for msg MUST not exceed
     * 3072 bytes.
     *  
     * @return String
     */
    public String getMsg();

    /**
     * Returns the priority.
     * 
     * Defines the importance of the event. The predefined priorities are:
     * 1. Low
     * 2. Medium
     * 3. High
     * The values are 0 to 100. The reserved value for Low is 10, for Medium is 50,
     * and for High is 70.  Other priorities MAY be added but MUST not exceed 100.
     * This is an OPTIONAL property.  
     * The default value is 0, which is a valid value for this property.
     * For this reason the '{@link org.eclipse.hyades.logging.events.ICommonBaseEvent#isSetPriority <em>isSetPriority()</em>}' 
     * method should always be called before calling this method.  This allows the user to 
     * distinguish the default value from a value that has been set to 0.

     * 
     * @return short
     */
    public short getPriority();

    /**
     * Returns the repeatCount.
     * 
     * The number of occurrences of a given event for a specific time interval.
     * This is an OPTIONAL property. A value of 0 or no value
     * is indicative of no repeat of the event detected.
     * The default value is 0, which is a valid value for this property.
     * For this reason the '{@link org.eclipse.hyades.logging.events.ICommonBaseEvent#isSetRepeatCount <em>isSetRepeatCount()</em>}' 
     * method should always be called before calling this method.  This allows the user to 
     * distinguish the default value from a value that has been set to 0.
     *
     * @return short
     */
    public short getRepeatCount();

    /**
     * Returns the sequenceNumber.
     * 
     * A source defined number that allows for multiple messages to be sent
     * and processed in logical order different from the order in which they 
     * arrived at consumer location (e.g., an event server or management tools).
     * The sequence number helps consumers to sort arrived messages. This is 
     * with respect to the creation time and to the particular reporter of the 
     * messages. This is an OPTIONAL property.  
     * The default value is 0, which is a valid value for this property.
     * For this reason the '{@link org.eclipse.hyades.logging.events.ICommonBaseEvent#isSetSequenceNumber <em>isSequenceNumber()</em>}' 
     * method should always be called before calling this method.  This allows the user to 
     * distinguish the default value from a value that has been set to 0.
     * 
     * @return long
     */
    public long getSequenceNumber();

    /**
     * Returns the severity.
     * 
     * The severity of the status the event is describing with respect to the 
     * application that reports the event.  The predefined severity levels, in
     * order of increasing severity, are as follows: 
     * 1. Unknown
     * 2. Information SHOULD be used for cases when the event only contains general
     *    information and is not reporting an error.
     * 3. Harmless SHOULD be used for cases when the error event has no effect on the
     *    normal operation of the resource.
     * 4. Warning SHOULD be used when it is appropriate to let the user decide if an
     *    action is needed in response to the event.
     * 5. Minor SHOULD be used to indicate that action is needed, but the situation
     *    is not serious at this time.
     * 6. Critical SHOULD be used to indicate that an immediate action is needed and
     *    the scope is broad (perhaps an imminent outage to a critical resource will 
     *    result).
     * 7. Fatal SHOULD be used to indicate that an error occurred, but it is too late
     *    to take remedial action.
     *   
     * The values are 0 to 70. The reserved values start at 0 for Unknown and increase
     * by increments of 10 to 60 for Fatal.  Other severities MAY be added but MUST not
     * exceed 70.  This is an OPTIONAL property.
     * The default value is 0, which is a valid value for this property.
     * For this reason the '{@link org.eclipse.hyades.logging.events.ICommonBaseEvent#isSetSeverity <em>isSetSeverity()</em>}' 
     * method should always be called before calling this method.  This allows the user to 
     * distinguish the default value from a value that has been set to 0.
     * 
     * @return short
     */
    public short getSeverity();

    /**
     * Returns the situationType.
     * 
     * The situationType specifies the type of the situation that caused the event 
     * to be reported. This is an extensible string value. Proposed reserved keywords
     *  are:
     * 1. START
     * 2. STOP
     * 3. FEATURE
     * 4. DEPENDENCY
     * 5. REQUEST
     * 6. CONFIGURE
     * 7. CONNECT
     * 8. CREATE
     * 9. UNKNOWN
     * This is an OPTIONAL property. The maximum string length for situationType MUST
     * not exceed 64 bytes.
     * NOTE: The situations type is being defined in parallel to the CommonBaseEventImpl
     * effort in a separate document, Canonical Situations ' ACAB.CT0304.0.1. ' The type
     * and the values of situationType are subject to change.
     * 
     * @return String The returned situationType.
     * @since 1.0 
     * @deprecated Use the ISituation as of 1.0.1.
     */
    public String getSituationType();

    /**
     * Returns the sourceComponentId.
     * 
     * Identification of the component that is 'affected' or was 'impacted'
     * by the event or the situation.  This is a REQUIRED property for the 
     * component that is affected by the situation. 
     * 
     * @return ComponentIdentificationImpl
     */
    public IComponentIdentification getSourceComponentId();

    /**
     * Returns the msgDataElement.
     * 
     * Identification of the message that this event holds.  See the
     * MsgDataElementImpl definition (described on page 30). This is an OPTIONAL
     * property.
     * 
     * @return MsgDataElementImpl
     */
    public IMsgDataElement getMsgDataElement();

    /**
     * Returns the reporterComponentId.
     * 
     * Identification of the component that is the 'reporter' of the event or the
     * situation. It is a REQUIRED property if the reporting component is different
     * than the source component.  Otherwise this field MUST be omitted. 
     * 
     * @return ComponentIdentificationImpl
     */
    public IComponentIdentification getReporterComponentId();

    /**
     * Returns the situation.
     * 
     * The situation specifies the type of the situation that caused the 
     * event to be reported. 
     * This is an REQUIRED property.
     * 
     * @return String The returned situation.
     * @since 1.0.1
     */
    public ISituation getSituation();

    /**
     * Returns the otherData.
     * 
     * @param String[] The returned otherData.
     * @since 1.0.1
     */
    public String[] getOtherData();

    /**
     * Returns the Common Base Event implementation version.
     * 
     * A string identifying the version of this event.  This field is OPTIONAL 
     * in that if it isnt specified, the version of the CBE is said to be 1.0.  
     * Otherwise this field MUST be filled in and MUST be consistent with the 
     * version specified in the schema header.The string length for version MUST 
     * NOT exceed 16 characters. When an application is creating an event following 
     * this version of the specification this field should be set to 1.0.1.
     * 
     * @return String The version of the Common Base Event implementation.
     * @deprecated Use getVersion() as of 1.0.1.
     * @see ICommonBaseEvent#VERSION_1_0
     * @see ICommonBaseEvent#VERSION_1_0_1
     */
    public String getCommonBaseEventImplVersion();

    /**
     * Returns the Common Base Event implementation version.
     * 
     * A string identifying the version of this event.  This field is OPTIONAL 
     * in that if it isnt specified, the version of the CBE is said to be 1.0.  
     * Otherwise this field MUST be filled in and MUST be consistent with the 
     * version specified in the schema header.The string length for version MUST 
     * NOT exceed 16 characters. When an application is creating an event following 
     * this version of the specification this field should be set to 1.0.1.
     * 
     * @return String The version of the Common Base Event implementation.
     * @see ICommonBaseEvent#VERSION_1_0
     * @see ICommonBaseEvent#VERSION_1_0_1
     */
    public String getVersion();

    /**
     * Returns the Common Base Event XML schema version.
     * 
     * @return String The version of the Common Base Event XML schema.
     * @deprecated Use getVersion() as of 1.0.1.
     * @see ICommonBaseEvent#VERSION_1_0
     * @see ICommonBaseEvent#VERSION_1_0_1
     */
    public String getEventSchemaVersion();

    /**
     * Returns the list of supported Common Base Event implementation version(s).
     * 
     * The list of supported Common Base Event implementation version(s) are all the 
     * current and previous supported version(s) of the Common Base Event implementation.
     * 
     * A string identifying the version of this event.  This field is OPTIONAL 
     * in that if it isnt specified, the version of the CBE is said to be 1.0.  
     * Otherwise this field MUST be filled in and MUST be consistent with the 
     * version specified in the schema header.The string length for version MUST 
     * NOT exceed 16 characters. When an application is creating an event following 
     * this version of the specification this field should be set to 1.0.1.
     * 
     * @return List The returned list of supported Common Base Event implementation version(s).
     * @since 1.0.1
     * @see ICommonBaseEvent#VERSION_1_0
     * @see ICommonBaseEvent#VERSION_1_0_1
     */
    public List getVersions();

    /**
     * Sets the creationTime.
     * 
     * The date-time when the event was issued. The value MUST be as defined by 
     * the XML Schema dateTime data type. The value of the creationTime MUST provide
     * granularity as precisely as the generating platform allows. This is a REQUIRED
     * property.
     * 
     * @param creationTime The creationTime to set
     */
    public void setCreationTime(String creationTime);

    /**
     * Sets the creationTime to a long that contains the milliseconds since Jan 1, 1970.
     * 
     * @param creationTime The creationTime to set
     */
    public void setCreationTime(long creationTime);

    /**
     * Sets the elapsedTime.
     * 
     * This is the time interval or the elapsed time for the number of occurrences
     * of a given event type that is specified by the repeatCount property. This
     * value indicates the duration of the time within which the repeated events 
     * were observed. This is an OPTIONAL property.  However, 
     * if repeatCount is specified you MUST specify a value for elapsedTime. 
     * The default value is 0, which is a valid value for this property.
     * Use the '{@link org.eclipse.hyades.logging.events.ICommonBaseEvent#isSetElapsedTime <em>isSetElapsedTime()</em>}' 
     * to distinguish the default value from a value that has been set to 0. 
     * 
     * @param elapsedTime The elapsedTime to set
     */
    public void setElapsedTime(long elapsedTime);

    /**
     * Sets the extensionName.
     * 
     * The name of an event class (or element in XML) that this event represents
     * (e.g., CommonBaseEventImpl). The name is indicative of any additional elements
     * expected to be present within the event. This is a OPTIONAL property.  If 
     * the value specified is null, then its value is assumed to be 'CommonBaseEventImpl'.
     * The maximum string length for extensionName MUST not exceed 64 bytes.
     * 
     * @param extensionName The extensionName to set
     */
    public void setExtensionName(String extensionName);

    /**
     * Sets the gloablInstanceId.
     * 
     * The primary identifier for the event. This property MUST be globally unique
     * and MAY be used as the primary key for the event. This is an OPTIONAL property.
     * However, once this value is set it MUST never be changed. The recommend value 
     * is either a 128 bit or 256 bit Globally Unique Id (represented as hex string).
     * 
     * @param gloablInstanceId The gloablInstanceId to set
     */
    public void setGlobalInstanceId(String globalInstanceId);

    /**
     * Sets the localInstanceId.
     * 
     * A source supplied event identifier. There is no guarantee that this value 
     * is globally unique. This is an OPTIONAL property. The maximum string length
     * for localInstanceId MUST not exceed 64 bytes.
     * 	
     * @param localInstanceId The localInstanceId to set
     */
    public void setLocalInstanceId(String localInstanceId);

    /**
     * Sets the msg.
     * 
     * The text accompanying the event. This is typically the resolved message 
     * string in human readable format rendered for a specific locale. This is 
     * an OPTIONAL property.  The maximum string length for msg MUST not exceed
     * 3072 bytes. 
     * 
     * @param msg The msg to set
     */
    public void setMsg(String msg);

    /**
     * Sets the priority.
     * 
     * Defines the importance of the event. The predefined priorities are:
     * 1. Low
     * 2. Medium
     * 3. High
     * The values are 0 to 100. The reserved value for Low is 10, for Medium is 50,
     * and for High is 70.  Other priorities MAY be added but MUST not exceed 100.
     * This is an OPTIONAL property.  
     * The default value is 0, which is a valid value for this property.
     * Use the '{@link org.eclipse.hyades.logging.events.ICommonBaseEvent#isSetPriority <em>isSetPriority()</em>}' 
     * to distinguish the default value from a value that has been set to 0. 
     * 
     * @param priority The priority to set
     */
    public void setPriority(short priority);

    /**
     * Sets the repeatCount.
     * 
     * The number of occurrences of a given event for a specific time interval.
     * This is an OPTIONAL property. A value of 0 or no value
     * is indicative of no repeat of the event detected.
     * The default value is 0, which is a valid value for this property.
     * Use the '{@link org.eclipse.hyades.logging.events.ICommonBaseEvent#isSetRepeatCount <em>isSetRepeatCount()</em>}' 
     * to distinguish the default value from a value that has been set to 0. 
     * 
     * @param repeatCount The repeatCount to set
     */
    public void setRepeatCount(short repeatCount);

    /**
     * Sets the sequenceNumber.
     * 
     * A source defined number that allows for multiple messages to be sent
     * and processed in logical order different from the order in which they 
     * arrived at consumer location (e.g., an event server or management tools).
     * The sequence number helps consumers to sort arrived messages. This is 
     * with respect to the creation time and to the particular reporter of the 
     * messages. This is an OPTIONAL property.
     * The default value is 0, which is a valid value for this property.
     * Use the '{@link org.eclipse.hyades.logging.events.ICommonBaseEvent#isSetSequenceNumber <em>isSetSequenceNumber()</em>}' 
     * to distinguish the default value from a value that has been set to 0. 
     * 
     * @param sequenceNumber The sequenceNumber to set
     */
    public void setSequenceNumber(long sequenceNumber);

    /**
     * Sets the severity.
     * 
     * The severity of the status the event is describing with respect to the 
     * application that reports the event.  The predefined severity levels, in
     * order of increasing severity, are as follows: 
     * 1. Unknown
     * 2. Information SHOULD be used for cases when the event only contains general
     *    information and is not reporting an error.
     * 3. Harmless SHOULD be used for cases when the error event has no effect on the
     *    normal operation of the resource.
     * 4. Warning SHOULD be used when it is appropriate to let the user decide if an
     *    action is needed in response to the event.
     * 5. Minor SHOULD be used to indicate that action is needed, but the situation
     *    is not serious at this time.
     * 6. Critical SHOULD be used to indicate that an immediate action is needed and
     *    the scope is broad (perhaps an imminent outage to a critical resource will 
     *    result).
     * 7. Fatal SHOULD be used to indicate that an error occurred, but it is too late
     *    to take remedial action.
     *   
     * The values are 0 to 70. The reserved values start at 0 for Unknown and increase
     * by increments of 10 to 60 for Fatal.  Other severities MAY be added but MUST not
     * exceed 70.  This is an OPTIONAL property.
     * The default value is 0, which is a valid value for this property.
     * Use the '{@link org.eclipse.hyades.logging.events.ICommonBaseEvent#isSetSeverity <em>isSetSeverity()</em>}' 
     * to distinguish the default value from a value that has been set to 0. 
     * 
     * @param severity The severity to set
     */
    public void setSeverity(short severity);

    /**
     * Sets the situationType.
     * 
     * The situationType specifies the type of the situation that caused the event 
     * to be reported. This is an extensible string value. Proposed reserved keywords
     *  are:
     * 1. START
     * 2. STOP
     * 3. FEATURE
     * 4. DEPENDENCY
     * 5. REQUEST
     * 6. CONFIGURE
     * 7. CONNECT
     * 8. CREATE
     * 9. UNKNOWN
     * This is an OPTIONAL property. The maximum string length for situationType MUST
     * not exceed 64 bytes.
     * NOTE: The situations type is being defined in parallel to the CommonBaseEventImpl
     * effort in a separate document, Canonical Situations ' ACAB.CT0304.0.1. ' The type
     * and the values of situationType are subject to change.
     * 
     * @param situationType The situationType to set
     * @since 1.0
     * @deprecated Use the ISituation as of 1.0.1.
     */
    public void setSituationType(String situationType);

    /**
     * Sets the sourceComponentId.
     * 
     * Identification of the component that is 'affected' or was 'impacted'
     * by the event or the situation.  This is a REQUIRED property for the 
     * component that is affected by the situation. 
     * 
     * @param sourceComponentId The sourceComponentId to set
     */
    public void setSourceComponentId(IComponentIdentification sourceComponentId);

    /**
     * Sets the msgDataElement.
     * 
     * Identification of the message that this event holds.  See the
     * MsgDataElementImpl definition (described on page 30). This is an OPTIONAL
     * property.
     * 
     * @param msgDataElement The msgDataElement to set
     */
    public void setMsgDataElement(IMsgDataElement msgDataElement);

    /**
     * Sets the reporterComponentId.
     * 
     * Identification of the component that is the 'reporter' of the event or the
     * situation. It is a REQUIRED property if the reporting component is different
     * than the source component.  Otherwise this field MUST be omitted. 
     * 
     * @param reporterComponentId The reporterComponentId to set
     */
    public void setReporterComponentId(IComponentIdentification reporterComponentId);

    /**
     * Sets the situation.
     * 
     * The situation specifies the type of the situation that caused the 
     * event to be reported. 
     * This is an REQUIRED property.
     * 
     * @param situation The situation to be set.
     * @since 1.0.1
     */
    public void setSituation(ISituation situation);

    /**
     * Sets the otherData.
     * 
     * @param situation The otherData to be set.
     * @since 1.0.1
     */
    public void setOtherData(String[] otherData);

    /**
     * Sets the preferredVersion.
     * 
     * The preferred version of the Common Base Event implementation as desired by the user.
     * 
     * A string identifying the version of this event.  This field is OPTIONAL 
     * in that if it isnt specified, the version of the CBE is said to be 1.0.  
     * Otherwise this field MUST be filled in and MUST be consistent with the 
     * version specified in the schema header.The string length for version MUST 
     * NOT exceed 16 characters. When an application is creating an event following 
     * this version of the specification this field should be set to 1.0.1.
     * 
     * @param preferredVersion The preferredVersion to be set.
     * @throws UnsupportedVersionException The preferredVersion is not a valid version.
     * @since 1.0.1
     * @see ICommonBaseEvent#VERSION_1_0
     * @see ICommonBaseEvent#VERSION_1_0_1
     */
    public void setPreferredVersion(String preferredVersion) throws UnsupportedVersionException;

    /**
     * Resets the object's properties to their initial (e.g. null) state.
     * 
     * Initialize a CommonBaseEvent so that it has a new guid and that all the 
     * following relationships have been cleared:
     * 1. IAssociatiatedEvent(s)
     * 2. IExtendeddataElement(s)
     * 3. IContextDataElement(s)
     * 
     * All components are initialized to either zero or null
     * @since 1.0
     */
    public void init();

    /**
     * This property allows for the grouping of events. This property is a
     * complex type that is made up of globalInstanceIds identifying the associated
     * events along with a type field describing the type of association that is
     * represented by the name of the association. Reserved keyword values include:
     * 1. Contain 
     * 2. CausedBy
     * 3. Cleared
     * 4. MultiPart
     * 5. Correlated
     * This is an OPTIONAL property.
     */
    public IAssociatedEvent addAssociatedEvent(String associationEngineName, String associationEngineType, String associationEngineId, String[] resolvedEvents);

    /**
     * Add a context that this event is referencing. See the
     * ContextDataElement definition (described on page 28) for details.
     * This is an OPTIONAL property.
     */
    public IContextDataElement addContextDataElementWithId(String contextType, String contextName, String contextId);

    /**
     * Add a context that this event is referencing. See the
     * ContextDataElement definition (described on page 28) for details.
     * This is an OPTIONAL property.
     */
    public IContextDataElement addContextDataElementWithValue(String contextType, String contextName, String contextValue);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElement(String name, int type, String value);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElement(String name, int type, String[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElement(String name, String value);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElement(String name, String[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithByteArrayValue(String name, byte[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithByteValue(String name, byte value);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithBooleanArrayValue(String name, boolean[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithBooleanValue(String name, boolean value);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithDateArrayValue(String name, String[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithDateAsLongValue(String name, long value);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithDatesAsLongValue(String name, long[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithDateValue(String name, String value);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithDoubleArrayValue(String name, double[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithDoubleValue(String name, double value);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithFloatArrayValue(String name, float[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithFloatValue(String name, float value);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithHexValue(String name, byte[] hexValue);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithHexValue(String name, String hexValue);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithIntArrayValue(String name, int[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithIntValue(String name, int value);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithLongArrayValue(String name, long[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithLongValue(String name, long value);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithShortArrayValue(String name, short[] values);

    /**
     * An array of product specific extensions for extensibility where you can
     * provide any other attributes not accounted for in the CommonBaseEventImpl.  
     * Information placed here is assumed to be product specific data.  This is
     * an OPTIONAL property.
     */
    public IExtendedDataElement addExtendedDataElementWithShortValue(String name, short value);

    /**
     * Checks if the specified property is set.
     */
    public boolean isSetCreationTime();

    /**
     * Checks if the specified property is set.
     */
    public boolean isSetElapsedTime();

    /**
     * Checks if the specified property is set.
     */
    public boolean isSetPriority();

    /**
     * Checks if the specified property is set.
     */
    public boolean isSetRepeatCount();

    /**
     * Checks if the specified property is set.
     */
    public boolean isSetSequenceNumber();

    /**
     * Checks if the specified property is set.
     */
    public boolean isSetSeverity();

    /**
     * Unsets the specified property is set.
     */
    public void unsetCreationTime();

    /**
     * Unsets the specified property is set.
     */
    public void unsetElapsedTime();

    /**
     * Unsets the specified property is set.
     */
    public void unsetPriority();

    /**
     * Unsets the specified property is set.
     */
    public void unsetRepeatCount();

    /**
     * Unsets the specified property is set.
     */
    public void unsetSequenceNumber();

    /**
     * Unsets the specified property is set.
     */
    public void unsetSeverity();

    /**
     * Sets the msgDataElement.
     * 
     * Identification of the message that this event holds.  See the
     * MsgDataElementImpl definition (described on page 30). This is an OPTIONAL
     * property.
     */
    public IMsgDataElement setMsgDataElement(String msgCatalog, String msgCatalogId, IMsgCatalogToken[] tokens, String msgCatalogType, String msgId, String msgIdType, String msgCatalogLocale);

    /**
     * Sets the msgDataElement.
     * 
     * Identification of the message that this event holds.  See the
     * MsgDataElementImpl definition (described on page 30). This is an OPTIONAL
     * property.
     */
    public IMsgDataElement setMsgDataElement(String msgCatalog, String msgCatalogId, String[] tokens, String msgCatalogType, String msgId, String msgIdType, String msgCatalogLocale);

    /**
     * Sets the reporterComponentId.
     * 
     * Identification of the component that is the 'reporter' of the event or the
     * situation. It is a REQUIRED property if the reporting component is different
     * than the source component.  Otherwise this field MUST be omitted. 
     */
    public IComponentIdentification setReporterComponentId(String application, String component, String subComponent, String componentType, String componentIdType, String location, String locationType);

    /**
     * Sets the reporterComponentId.
     * 
     * Identification of the component that is the 'reporter' of the event or the
     * situation. It is a REQUIRED property if the reporting component is different
     * than the source component.  Otherwise this field MUST be omitted. 
     */
    public IComponentIdentification setReporterComponentId(String application, String component, String subComponent, String componentType, String componentIdType, String executionEnvironment, String instanceId, String location, String locationType, String processId, String threadId);

    /**
     * Sets the sourceComponentId.
     * 
     * Identification of the component that is the 'source' of the event or the
     * situation. It is a REQUIRED property if the reporting component is different
     * than the source component.  Otherwise this field MUST be omitted. 
     */
    public IComponentIdentification setSourceComponentId(String application, String component, String subComponent, String componentType, String componentIdType, String location, String locationType);

    /**
     * Sets the sourceComponentId.
     * 
     * Identification of the component that is the 'source' of the event or the
     * situation. It is a REQUIRED property if the reporting component is different
     * than the source component.  Otherwise this field MUST be omitted. 
     */
    public IComponentIdentification setSourceComponentId(String application, String component, String subComponent, String componentType, String componentIdType, String executionEnvironment, String instanceId, String location, String locationType, String processId, String threadId);
}
